/*
 * Access a calculation server.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>

#include <stdio.h>

#include "calc_client.h"
#include "calc_local.h"

/* A generic function that can invoke local and remote objects. */

static void invoke(Calc *obj, int *result)
{
  obj->add(789, 123, result);
}

/* Main program. */

int main(void)
{
  l4_cap_idx_t server = l4re_env_get_cap("server");

  /* Calculator encapsulation. */

  client_Calc obj(server);

  /* Another calculator. */

  local_Calc local_obj;

  /* Call the calculator. */

  int result;

  /* Invoke the operation via the interface. */

  invoke(&obj, &result);
  printf("add 789, 123 -> %d\n", result);

  /* Invoke an operation on the local object via the interface. */

  invoke(&local_obj, &result);
  printf("add 789, 123 -> %d\n", result);

  return 0;
}
